/*********************************************************************
* <pre>
* LL   VV  VV LL   7777777  (C) Copyright LVL7 Systems 2000-2006
* LL   VV  VV LL   7   77   All Rights Reserved.
* LL   VV  VV LL      77
* LL    VVVV  LL     77
* LLLLL  VV   LLLLL 77      Code classified LVL7 Confidential
* </pre>
**********************************************************************
**********************************************************************
*
* @filename     update_image.c
*
* @purpose      dual image file utility
*
* @component    dim
*
* @comments     Stand alone utility called by rc.fastpath script to
#               read the contents of the dual image configuration file.
*
* @create       14/03/2008
*
* @author       saleem
* @end
*
**********************************************************************/

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <stdint.h>
#include <mtd/mtd-abi.h>
#include <sys/ioctl.h>

#include "dim.h"

#define STK_TAG1                ((unsigned short) 0xaa55)
#define STK_TAG2                ((unsigned long) 0x2288bb66)
#define MAX_IMAGE_SIZE          (6 * 1024 * 1024)
#define IMG_MTD_DEVICE          "/dev/mtd4"
#define DOWNLOAD_PATH           "/mnt/download/"

typedef struct
{
        unsigned short crc;
        unsigned short tag1;
        unsigned long tag2;

        unsigned long num_components; /* Number of OPR and tgz files in the STK image (may be 0) */

        unsigned long file_size; /* Total number of bytes in the STK file */

        unsigned char rel;
        unsigned char ver;
        unsigned char maint_level;
        unsigned char build_num;

        unsigned long stk_header_size; /* Number of bytes in the STk header */
        unsigned char reserved[76];    /* Reserved for future use */
} stkFileHeader_t;
/* ===================== Function Definitions ================ */

void usage(void) {
  printf("\nusage: update_image [<options>]*\n\n"
	 "Where <options> can be:\n\n"
	 "  -image1        write image1 to flash\n"
	 "  -image2        write image2 to flash\n\n");
  exit(0);
};

int main(int argc, char* argv[]) {
  
  int arg=1;
  int  rc = 1;  /* Default value is image1 */

  if((argc==1) || (argc > 2)) {
    usage();
    exit(0);
  };

  if (strcmp("-image1", argv[1]) == 0)
  {
    return transfer_process_code(DIM_IMAGE1_NAME);
  }
  else if (strcmp("-image2", argv[1]) == 0)
  {
    return transfer_process_code(DIM_IMAGE2_NAME);
  } 
  else
  {
    usage();
    exit(0);
  }
  return rc;
};

int transfer_process_code(char *fileName)
{
  /* We're only called on one task's thread; use static buffers to save
     stack space */
  char buf[0x80];
  stkFileHeader_t stk_header;
  struct erase_info_user mtdEraseInfo;
  char full_fname[128];
  int fd,rc;

  /* move the file to the CONFIG_PATH */
  snprintf(full_fname, sizeof(full_fname), "%s%s",DOWNLOAD_PATH,fileName);

  if ((fd = open(full_fname, O_RDWR, 0644)) == -1)
  {
    return 1;
  }


  if ( read (fd, (unsigned char *)&stk_header,
        sizeof(stkFileHeader_t)) < 0)
  { 
    close(fd);
    return 1;
  }

  if ( (stk_header.tag1 != STK_TAG1) || (stk_header.tag2 != STK_TAG2))
  {
    close(fd);
    return 1;
  }

  close(fd);
  fd = open(IMG_MTD_DEVICE, O_RDWR);

  if (fd < 0)
  {
    return 1;
  }

  if (stk_header.file_size > MAX_IMAGE_SIZE)
  {
    mtdEraseInfo.start = 0;
    mtdEraseInfo.length = (2 * MAX_IMAGE_SIZE);
    snprintf(buf, sizeof(buf), "dd if=%s/%s of=%s >/dev/null 2>&1",
             DOWNLOAD_PATH, fileName, IMG_MTD_DEVICE);
  }
  else if (strcmp(fileName, DIM_IMAGE1_NAME) == 0)
  {
    mtdEraseInfo.start = 0;
    mtdEraseInfo.length = MAX_IMAGE_SIZE;
    snprintf(buf, sizeof(buf), "dd if=%s/%s of=%s >/dev/null 2>&1",
        DOWNLOAD_PATH, fileName, IMG_MTD_DEVICE);
  }
  else
  {
    mtdEraseInfo.start = MAX_IMAGE_SIZE;
    mtdEraseInfo.length = MAX_IMAGE_SIZE;
    snprintf(buf, sizeof(buf), "dd if=%s/%s of=%s bs=128 seek=49152 >/dev/null 2>&1",
        DOWNLOAD_PATH, fileName, IMG_MTD_DEVICE);
  }
  ioctl(fd, MEMUNLOCK, &mtdEraseInfo);
  if (ioctl (fd, MEMERASE, &mtdEraseInfo) < 0)
  {
    close(fd);
    return 1;
  }
  close(fd);

  if (system(buf) != 0) 
  {
    return 1;
  }
  return 0;
}
