/*
 * Linux OS Independent Layer
 *
 * Copyright (C) 2004 Broadcom Corporation
 * Copyright (C) 2005 MontaVista Inc. ( 2.6 kernel modifications )
 * Copyright (C) 2007 Wind River Systems. ( Only keep LinuxOS code )
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * Linux OS Independent Layer
 *
 */

#ifndef _linux_osl_h_
#define _linux_osl_h_

#include <asm/brcm/typedefs.h>

/* use current 2.4.x calling conventions */
#include <asm/brcm/linuxver.h>

/* assert and panic */
#ifdef BCMDBG
#define ASSERT(exp) \
	do { if (!(exp)) osl_assert(#exp, __FILE__, __LINE__); } while (0)
extern void osl_assert(char *exp, char *file, int line);
#else
#define	ASSERT(exp)		do {} while (0)
#endif

/* PCMCIA attribute space access macros */
#define	OSL_PCMCIA_READ_ATTR(osh, offset, buf, size) \
	osl_pcmcia_read_attr((osh), (offset), (buf), (size))
#define	OSL_PCMCIA_WRITE_ATTR(osh, offset, buf, size) \
	osl_pcmcia_write_attr((osh), (offset), (buf), (size))
extern void osl_pcmcia_read_attr(void *osh, uint offset, void *buf,
				 int size);
extern void osl_pcmcia_write_attr(void *osh, uint offset, void *buf,
				  int size);

/* PCI configuration space access macros */
#define	OSL_PCI_READ_CONFIG(loc, offset, size) \
	osl_pci_read_config((loc), (offset), (size))
#define	OSL_PCI_WRITE_CONFIG(loc, offset, size, val) \
	osl_pci_write_config((loc), (offset), (size), (val))
extern uint32 osl_pci_read_config(void *loc, uint size, uint offset);
extern void osl_pci_write_config(void *loc, uint offset, uint size,
				 uint val);

/* OSL initialization */
#define osl_init()		do {} while (0)

/* host/bus architecture-specific byte swap */
#define BUS_SWAP32(v)		(v)

/*
 * Macros expand to calls to functions defined in linux_osl.c .
 */

/* string library */
#define	printf(fmt, args...)	printk(fmt, ## args)
#include <linux/kernel.h>
#include <linux/string.h>

/* register access macros */

#define R_REG(r) ({ \
	__typeof(*(r)) __osl_v; \
	switch (sizeof(*(r))) { \
	case sizeof(uint8):	__osl_v = readb((void*)(r)); break; \
	case sizeof(uint16):	__osl_v = readw((void*)(r)); break; \
	case sizeof(uint32):	__osl_v = readl((void*)(r)); break; \
	} \
	__osl_v; \
})

#define W_REG(r, v) do { \
	switch (sizeof(*(r))) { \
	case sizeof(uint8):	writeb((uint8)(v),(void*)(r)); break; \
	case sizeof(uint16):	writew((uint16)(v),(void*)(r)); break; \
	case sizeof(uint32):	writel((uint32)(v),(void*)(r)); break; \
	} \
} while (0)

#define	AND_REG(r, v)		W_REG((r), R_REG(r) & (v))
#define	OR_REG(r, v)		W_REG((r), R_REG(r) | (v))

/* bcopy, bcmp, and bzero */
#define	bcopy(src, dst, len)	memcpy((dst), (src), (len))
#define	bcmp(b1, b2, len)	memcmp((b1), (b2), (len))
#define	bzero(b, len)		memset((b), '\0', (len))

/* general purpose memory allocation */
#define	MALLOC(size)		kmalloc((size), GFP_ATOMIC)
#define	MFREE(addr, size)	kfree((addr))

/* uncached virtual address */
#ifdef mips
#define OSL_UNCACHED(va)	KSEG1ADDR((va))
#include <asm/addrspace.h>
#else
#define OSL_UNCACHED(va)	(va)
#endif

/* get processor cycle count */
#if defined(mips)
#define	OSL_GETCYCLES(x)	((x) = read_c0_count() * 2)
#elif defined(__i386__)
#define	OSL_GETCYCLES(x)	rdtscl((x))
#else
#define OSL_GETCYCLES(x)	((x) = 0)
#endif

/* map/unmap physical to virtual I/O */
#define	REG_MAP(pa, size)	ioremap_nocache((unsigned long)(pa), (unsigned long)(size))
#define	REG_UNMAP(va)		iounmap((void *)(va))

/* dereference an address that may cause a bus exception */
#ifdef mips
#if defined(MODULE) && (LINUX_VERSION_CODE < KERNEL_VERSION(2,4,17))
#define BUSPROBE(val, addr)	panic("get_dbe() will not fixup a bus exception when compiled into a module")
#else
#define	BUSPROBE(val, addr)	get_dbe((val), (addr))
#include <asm/paccess.h>
#endif
#else
#define	BUSPROBE(val, addr)	({ (val) = R_REG((addr)); 0; })
#endif

/* allocate/free shared (dma-able) consistent (uncached) memory */
#define	DMA_ALLOC_CONSISTENT(dev, size, pap) \
	pci_alloc_consistent((dev), (size), (dma_addr_t*)(pap))
#define	DMA_FREE_CONSISTENT(dev, va, size, pa) \
	pci_free_consistent((dev), (size), (va), (dma_addr_t)(pa))

/* map/unmap direction */
#define	DMA_TX			PCI_DMA_TODEVICE
#define	DMA_RX			PCI_DMA_FROMDEVICE

/* map/unmap shared (dma-able) memory */
#define	DMA_MAP(dev, va, size, direction, p) \
	pci_map_single((dev), (va), (size), (direction))
#define	DMA_UNMAP(dev, pa, size, direction, p) \
	pci_unmap_single((dev), (dma_addr_t)(pa), (size), (direction))

/* microsecond delay */
#define	OSL_DELAY(usec)		udelay(usec)
#include <linux/delay.h>
#define OSL_SLEEP(usec) set_current_state(TASK_INTERRUPTIBLE); \
                        schedule_timeout((usec*HZ)/1000000);
#define OSL_IN_INTERRUPT() in_interrupt()



/* shared (dma-able) memory access macros */
#define	R_SM(r)			*(r)
#define	W_SM(r, v)		(*(r) = (v))
#define	BZERO_SM(r, len)	memset((r), '\0', (len))

/* packet primitives */
#define	PKTGET(drv, len, send)		osl_pktget((drv), (len), (send))
#define	PKTFREE(drv, skb, send)		osl_pktfree((skb))
#define	PKTDATA(drv, skb)		(((struct sk_buff*)(skb))->data)
#define	PKTLEN(drv, skb)		(((struct sk_buff*)(skb))->len)
#define	PKTNEXT(drv, skb)		(((struct sk_buff*)(skb))->next)
#define	PKTSETNEXT(skb, x)		(((struct sk_buff*)(skb))->next = (struct sk_buff*)(x))
#define	PKTSETLEN(drv, skb, len)	__skb_trim((struct sk_buff*)(skb), (len))
#define	PKTPUSH(drv, skb, bytes)	skb_push((struct sk_buff*)(skb), (bytes))
#define	PKTPULL(drv, skb, bytes)	skb_pull((struct sk_buff*)(skb), (bytes))
#define	PKTDUP(drv, skb)		skb_clone((struct sk_buff*)(skb), GFP_ATOMIC)
#define	PKTCOOKIE(skb)			((void*)((struct sk_buff*)(skb))->csum)
#define	PKTSETCOOKIE(skb, x)		(((struct sk_buff*)(skb))->csum = (uint)(x))
#define	PKTLINK(skb)			(((struct sk_buff*)(skb))->prev)
#define	PKTSETLINK(skb, x)		(((struct sk_buff*)(skb))->prev = (struct sk_buff*)(x))
extern void *osl_pktget(void *drv, uint len, bool send);
extern void osl_pktfree(void *skb);

#endif				/* _linux_osl_h_ */
